@@ -3,11 +3,12 @@ import socket
 import functools
 
 from minio import Minio
+from typing import Optional
 
 from automon.log import Logging
 
 from .bucket import Bucket
-from .object import Object
+from .object import Object, DeleteObject
 from .config import MinioConfig
 from .assertions import MinioAssertions
 
@@ -41,33 +42,25 @@ class MinioClient(object):
         )
         return client
 
-    def _isConnected(func):
+    def _is_connected(func):
         """Decorator that checks if MinioClient is connected
         """
 
         @functools.wraps(func)
         def _wrapper(self, *args, **kwargs):
-            if self.config.isReady():
-                if self.client.list_buckets():
-                    # if not self._sessionExpired() or self.client.list_buckets():
-                    return func(self, *args, **kwargs)
+            if not self.config.isReady():
+                return False
+            try:
+                self.client.list_buckets()
+                # if not self._sessionExpired() or self.client.list_buckets():
+                return func(self, *args, **kwargs)
+            except Exception as e:
+                log.error(f'Minio client not connected. {e}')
+            return False
 
         return _wrapper
 
-    @_isConnected
-    def clear_bucket(self, bucket_name, folder=None):
-        bucket_name = MinioAssertions.bucket_name(bucket_name)
-        objects = self.list_objects(bucket_name, folder)
-
-        if objects:
-            objects = [Object(x) for x in objects]
-
-            log.debug(f'Bucket: {bucket_name} Objects: {len(objects)}')
-            return self.client.remove_objects(bucket_name, objects)
-
-        return False
-
-    @_isConnected
+    @_is_connected
     def download_object(self, bucket_name, file):
         """Minio object downloader
         """
@@ -76,19 +69,42 @@ class MinioClient(object):
         log.debug(f'[downloader] Downloading: {bucket_name}/{file.object_name}')
         return self.client.get_object(bucket_name, file.object_name)
 
-    @_isConnected
+    @_is_connected
+    def get_bucket(self, bucket_name: str) -> Optional[Bucket]:
+        """List Minio buckets"""
+        bucket_name = MinioAssertions.bucket_name(bucket_name)
+        buckets = self.list_buckets()
+
+        for b in buckets:
+            if b == bucket_name:
+                log.info(f'Bucket: "{b}"')
+                return b
+
+        log.error(msg=f'Bucket "{bucket_name}" does not exist', raise_exception=False)
+        return
+
+    @_is_connected
     def isConnected(self):
         """Check if MinioClient is connected
         """
         log.info(f'Minio client OK')
         return True
 
-    @_isConnected
-    def list_objects(self,
-                     bucket_name: str,
-                     folder: str = None,
-                     recursive: bool = True,
-                     **kwargs) -> [Object]:
+    @_is_connected
+    def list_buckets(self) -> [Bucket]:
+        """List Minio buckets"""
+        buckets = self.client.list_buckets()
+        buckets = [Bucket(x) for x in buckets]
+
+        log.info(f'Buckets total: {len(buckets)}')
+        return buckets
+
+    @_is_connected
+    def list_objects(
+            self,
+            bucket_name: str,
+            folder: str = None,
+            recursive: bool = True, **kwargs) -> [Object]:
         """List Minio objects"""
         bucket_name = MinioAssertions.bucket_name(bucket_name)
 
@@ -96,10 +112,31 @@ class MinioClient(object):
             objects = self.client.list_objects(bucket_name, folder, recursive=recursive, **kwargs)
             objects = [Object(x) for x in objects]
 
+            msg = f'Objects total: {len(objects)} (bucket: "{bucket_name}")'
+
             if folder:
-                log.debug(f'Bucket: {bucket_name}, Folder: {folder}, Objects: {len(objects)}')
-            else:
-                log.debug(f'Bucket: {bucket_name}, Objects: {len(objects)}')
+                msg += f' Folder: "{folder}"'
+
+            log.info(msg)
+            return objects
+
+        except Exception as e:
+            log.error(f'failed to list objects. {e}')
+
+        return
+
+    @_is_connected
+    def list_objects_generator(
+            self,
+            bucket_name: str,
+            folder: str = None,
+            recursive: bool = True,
+            **kwargs) -> [Object]:
+        """Generator for Minio objects"""
+        bucket_name = MinioAssertions.bucket_name(bucket_name)
+
+        try:
+            objects = self.client.list_objects(bucket_name, folder, recursive=recursive, **kwargs)
             return objects
 
         except Exception as e:
@@ -107,43 +144,52 @@ class MinioClient(object):
 
         return False
 
-    @_isConnected
-    def list_buckets(self) -> [Bucket]:
-        """List Minio buckets"""
-        buckets = self.client.list_buckets()
-        buckets = [Bucket(x) for x in buckets]
-
-        log.info(f'Buckets: {len(buckets)} {[x.name for x in buckets]}')
-        return buckets
-
-    @_isConnected
-    def get_bucket(self, bucket_name: str) -> Bucket:
-        """List Minio buckets"""
+    @_is_connected
+    def remove_bucket(self, bucket_name: str) -> Optional[bool]:
         bucket_name = MinioAssertions.bucket_name(bucket_name)
-        buckets = self.list_buckets()
 
-        for b in buckets:
-            if b == bucket_name:
-                log.info(f'Buckets: {len(buckets)} {[x.name for x in buckets]}')
-                return b
+        try:
+            self.client.remove_bucket(bucket_name)
+            log.info(f'Removed bucket "{bucket_name}"')
+            return True
 
-        log.error(msg=f'Bucket "{bucket_name}" does not exist', raise_exception=False)
-        return False
+        except Exception as e:
+            log.error(f'Remove bucket "{bucket_name}" failed. {e}', enable_traceback=False)
 
-    @_isConnected
+        return
+
+    @_is_connected
+    def remove_objects(self, bucket_name, folder=None) -> bool:
+        bucket_name = MinioAssertions.bucket_name(bucket_name)
+        objects = self.list_objects(bucket_name, folder)
+        delete_objects = [DeleteObject(x) for x in objects]
+
+        if not delete_objects:
+            log.info(f'Bucket is empty: "{bucket_name}"')
+            return
+
+        errors = list(self.client.remove_objects(bucket_name, delete_objects))
+        log.info(f'Removed {len(delete_objects)} objects in bucket "{bucket_name}"')
+
+        if self.list_objects(bucket_name, folder):
+            return self.remove_objects(bucket_name, folder=folder)
+
+        return True
+
+    @_is_connected
     def make_bucket(self, bucket_name: str) -> Bucket:
         """Make a bucket"""
         bucket_name = MinioAssertions.bucket_name(bucket_name)
         try:
             self.client.make_bucket(bucket_name)
-            log.debug(f'Created bucket: {bucket_name}')
+            log.info(f'Created bucket "{bucket_name}"')
 
         except Exception as e:
-            log.error(f'Bucket exists: {bucket_name}', enable_traceback=False)
+            log.warn(f'Bucket exists: "{bucket_name}"')
 
         return self.get_bucket(bucket_name)
 
-    @_isConnected
+    @_is_connected
     def put_object(self, bucket_name: str, object_name: str, data: io.BytesIO, length: int = None,
                    content_type='application/octet-stream',
                    metadata=None, sse=None, progress=None,
@@ -182,20 +228,6 @@ class MinioClient(object):
 
             return False
 
-    @_isConnected
-    def remove_bucket(self, bucket_name: str) -> bool:
-        bucket_name = MinioAssertions.bucket_name(bucket_name)
-
-        try:
-            self.client.remove_bucket(bucket_name)
-            log.info(f'Removed bucket: {bucket_name}')
-            return True
-
-        except Exception as e:
-            log.error(f'Bucket does not exist: {bucket_name} {e}', enable_traceback=False)
-
-        return False
-
 
 def check_connection(host, port):
     try:
